#ifndef LOAN_H
#define LOAN_H

#include <FL/Fl_Check_Button.H>
#include "Borrower.h"
#include "Lender.h"
#include "Bundle.h"
#include "Selection.h"
#include <stdio.h>
#include "SQL_UTIL.h"
#include <fstream>

class Loan : public Fl_Check_Button {
 
 public:
  int loanID;				//Loan ID
  int candidate;			//Indicator for candidate bundle
  bool bundled;				//Indicator for bundle
  float originalValue;			//Original Principal of Loan
  float principalRemaining;		//Remaining Principal
  float profit;				//Amount of interest to be earned
  float discountRate;			//Amount loan is sold for
  float interestRate;			//Annual Interest Rate
  bool isARM;				//Indicator for interest type
  char* settlementDate;			//Settlement Date of Loan
  int term;				//Term in months
  int risk;				//Risk of the Loan
  int status;				//Loan status (good, late, default)
  int lenderID;				//Lender ID
  bool isJumbo;				//Indicator for loan size
  double propertyID;			//Property ID for loan
  char* buttonlabel;			//Label for GUI output

  //*****************************displayLoans**********************
  //Creates the dynamic SQL statement that will be passed to the 
  //get_all_loans function (based on selection criteria from GUI). 
  //Returns pointer to head of linkedlist or NULL if no loans found matching
  //the search criteria.  Parameters are selection criteria (from GUI),
  //and position in the scroll-window (handled by GUI).
  //***************************************************************
  static Loan* displayLoans( Selection* , int);

  Loan *prev;				//Previous loan in linked list
  Loan *next;				//Next loan in linked list

  //**************************Constructor**************************
  //Creates a loan, subclassed from the Fl_Check_Button class provided
  //by the FLTK library. Parameters are all data members as well as
  //a pointer to the previous loan in the linked list. The constructor
  //also sets the buttonlabel data member using given loan info.
  //***************************************************************
  Loan( int loan_id, int lender_id, float original_amount, float prin_remain,
        float int_rate, bool rate_type,char* sett_date, bool loan_type,
        int loan_term, int loan_status, int cand, bool bun, int loan_risk, 
	float loan_profit,float disc_rate, Loan* previous ) 
    : Fl_Check_Button( 15, 75, 400, 75, " " )     {
    
    loanID = loan_id;
    lenderID = lender_id;
    originalValue=original_amount;
    principalRemaining = prin_remain;
    interestRate = int_rate;
    isARM = rate_type;
    status = loan_status;

    settlementDate = new char[40];
    strcpy(settlementDate,sett_date);

    isJumbo = loan_type;
    term = loan_term;
    candidate = cand;
    bundled = bun;
    risk = loan_risk;
    profit = loan_profit;
    discountRate = disc_rate;

    prev = previous;
    next = NULL;

    char irtype[4];

    if( rate_type ) {
      strcpy(irtype,"FM");
    } else {
      strcpy(irtype,"ARM");
    }

    char loansize[8];

    if( isJumbo ) {
      strcpy( loansize, "Regular" ); // logic reversed
    } else {
      strcpy( loansize, "Jumbo" );
    }
    
    char* l_status;
    if(status==0) l_status = "Good";
    if(status==1) l_status = "Late";
    if(status==2) l_status = "Default";

    char *buttonlabel = new char[500];
    if( candidate != -1 ) { //If the loan is in a candidate or final bundle
      sprintf( buttonlabel, "UserID:%d   ID:%d   LenderID:%d  Value:$%.2f\n"
                            "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                            "Settlement Date:%s   Term:%d   Status=%s\n"
                            "DiscountRate:$%.2f   PrincipalRem:$%.2f",
               candidate, loanID, lenderID, originalValue, loansize, risk,
               interestRate, irtype, settlementDate, term, l_status,
               discountRate, principalRemaining                              );
      label(buttonlabel);

      if( bundled ) {
	labelcolor(FL_DARK_GREEN); //Bundled loans are green
      } else {
	labelcolor(FL_DARK_RED); //Candidate loans are red
      }
    } else { //Loan is not in a candidate or final bundle

      sprintf( buttonlabel, "ID:%d   LenderID:%d   Value:$%.2f\n"
                            "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                            "Settlement Date:%s   Term:%d  Status=%s \n"
                            "DiscountRate:$%.2f   PrincipalRem:$%.2f",
               loanID, lenderID, originalValue, loansize, risk,
               interestRate, irtype, settlementDate, term, l_status, 
	       discountRate, principalRemaining                              );

      label(buttonlabel);
    }

  };

  //******************************Default Constructor*********************
  //The default constructor simply creates a loan, subclassed as a check
  //button from the GUI.  It sets the next and previous pointers for the
  //linked list to NULL.
  //**********************************************************************
  Loan() : Fl_Check_Button(15,65,400,50,"") { 

    next = NULL;
    prev = NULL;

  };

  //****************************writeLoanReport***************************
  //This function takes care of writing a loan report file.  Its parameter
  //is the head of the linked list of loans to write the report on.
  //**********************************************************************
  void writeLoanReport(Loan* list){

    ofstream fout("loans_report.txt");
    char output[500];

    while(list!=NULL){

      char irtype[4];

      if( !list->isARM ) {
        strcpy(irtype,"FM");
      } else {
        strcpy(irtype,"ARM");
      }

      char loansize[8];

      if( list->isJumbo ) {
        strcpy( loansize, "Regular" ); // logic reversed
      } else {
        strcpy( loansize, "Jumbo" );
      }
      sprintf( output, "ID:%d   LenderID:%d   Value:$%.2f\n"
                       "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                       "Settlement Date:%s   Term:%d\n"
                       "DiscountRate:$%.2f   PrincipalRem:$%.2f\n",
               list->loanID, list->lenderID, list->originalValue,
	       loansize, list->risk, list->interestRate, irtype,
               list->settlementDate, list->term, list->discountRate,
               list->principalRemaining                              );
      
      fout << output << endl;
      list = list->next;
    }
    fout.close();
  }

};

//***************************get_selected_loans******************
//This function takes care of actually retrieving loans from the
//database. Its parameters are the dynamic SQL statement, which
//is created by displayLoans, and the offset in the scroll window,
//also passed through displayLoans. It returns a loan pointer to
//the head of the linked list to be displayed.
//*************************************************************** 
Loan* get_selected_loans( const char*,int);


//***************************create_bundle***********************
//This function inserts a loan bundle into the DB with a unique
//bundle ID. Parameters are investor name, bundle cost, bundle
//risk, and bundle profit. Returns the new bundle ID.
//***************************************************************
int create_bundle(char*,float,int,float);


//*************************bundle_loans**************************
//This function uses the bundle id from the create_bundle function
//to mark all selected loans as being in the bundle. Parameters are
//the linkedlist of loans and the bundle id to be assigned.
//***************************************************************
void bundle_loans(Loan*,int);


//***********************optimize*******************************
//Optimize function takes in a linked list of loans and chooses
//a subset as an "optimal" bundle. It then returns the linked
//list with the subset loans checked. Parameters are the linked
//list, max term, min profit, and max cost. Our optimize function
//uses a heuristic to choose loans that are more costly and less
//profitable.
//**************************************************************
Loan* optimize(Loan*,int,int,float,float);


//**********************getBundles******************************
//This function grabs all of the bundles in the DB and creates
//a linked list of them as bundle objects. It takes no params
//and simply returns the linked list it creates.
//**************************************************************
Bundle* getBundles();


//*********************displayCandLoans*************************
//This function grabs only the candidate loans of the current user
//from the DB. It then creates a linkedlist of loan objects, and
//returns the head of that list. 
//**************************************************************
Loan* displayCandLoans();


//********************mark_loan*********************************
//This function marks an individual loan in the DB as being chosen
//for inclusion in a candidate bundle.  It takes in the loan
//ID as a parameter, and returns true if the loan was able to be
//bundled, or false if the loan has already been marked by
//another loan analyst.
//**************************************************************
bool mark_loan(int);


//*********************unmark_loan******************************
//Basically the opposite of the previous function, this function
//unmarks a loan, removing it from a candidate bundle. It takes
//the loan ID as a parameter, and does not return a value.
//**************************************************************
void unmark_loan(int);
#endif


