//****************************************************************************
// MODULE NAME: Reports
//
// DESCRIPTION:  Reads in New Loans Report and Loan Status Report, interfaces
//   with Oracle Database and updates info.
//
//****************************************************************************
#ifndef report_hpp
#define report_hpp

#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <iomanip>
#include <unistd.h>
#include <sys/wait.h>



class Reports {

 public:
   void new_loans_report( void );
   //*******************************************************************
   // Params: None.
   //
   // Inputs: New Loans Report File
   //
   // Outputs: writes to database
   //
   // Description: Reads the New Loans Report and updates both Loan
   //   Portfolio and Borrower Database, New Lenders must be added first
   //   in the Lender CRUD.
   //*******************************************************************
 
   void loan_status_report( void );
   //*******************************************************************
   // Params: None.
   //
   // Inputs: Loan Status Report File
   //
   // Outputs: Updates Portfolio and Borrower Database
   //
   // Description: Reads the Loan Status Report and passes new info to
   //   Loan Database, Database will then update Status and Borrower 
   //   Risk as necessary.
   //*******************************************************************

   bool verified( int );
   //*******************************************************************
   // Params: int-enum DRY_RUN for initial run through to find errors
   //                  GAME_TIME for fixing errors or skipping records
   //
   // Inputs: None
   //
   // Outputs: None
   //
   // Return: bool - true for record is valid
   //              - false for invalid fields in record
   //
   // Description: Checks each field for -1's signifying report could
   //              not be interpreted for that field.  During DRY_RUN
   //              it returns true if record is valid and false if it
   //              contains errors.  For GAME_TIME returns true if
   //              record is valid and if is invalid brings up the 
   //              FixInfo Dialog Box.  If record is repaired returns
   //              true.  If record still contains errors, returns false.
   //*******************************************************************

   enum {
     DRY_RUN,   // checking entire report for errors, not fixing
     GAME_TIME  // adding or updating loans now, so fix bad records
   };
     
   enum {
     GOOD = 0,   // Good standing
     LATE = 1,   // Late standing
     DEFAULT = 2 // Default standing
   };

   enum {
     FM = 0, // Fixed Rate
     ARM = 1 // Adjustable Rate
   };

   enum {
     JUMBO = 0,  // Jumbo Loan $250000+
     REGULAR = 1 // Regular Loan $249999.99-
   };

   int Record_Type;             // 0-99
   double Loan_ID;              // Loan ID 0-9999999999
   double Lender_ID;            // Lender ID 0-99999999
   int FMCO_Purch;              // FMCO Purchase 0/1
   double Original_Value;       // Original Value 1000-500000
   double Interest;             // Expected total Interest Payed 
   float Interest_Rate;         // Interest Rate 00.001-99.999
   int Rate_Type;               // FM or ARM
   double ARM_Principal;        // Principal Remaining 1000-500000
   int MM;                      // Month 1-12
   int DD;                      // Day 1-31
   int YYYY;                    // Year no range
   int Term;                    // Term 1-30
   int Loan_Type;               // Loan Type JUMBO/REGULAR
   double Property_ID;          // Property ID 0-9999999999
   int Loan_Status;             // GOOD/LATE/DEFAULT
   double Monthly_Payment;      // Used to calc PR and Discount Rate for loans
   double Borrower_ID[4];       // 0-9999999999
   char Borrower_Name[4][65];   // Name

 private:
   int get_int( char *token, int min, int max );
   //*****************************************************************
   // Params: token - string token
   //         min - minimum value of int
   //         max - maximum value of int
   // Inputs: None.
   // Outputs: None.
   // Return Value: int - integer value from token, -1 if outside of
   //                     range
   // Description: Parses token for an integer within the min and max 
   //   values, if outside of range -1 is returned instead.
   //****************************************************************

   double get_double( char*, double, double );
   //*****************************************************************
   // Params: token - string token
   //         min - minimum value of double
   //         max - maximum value of double
   // Inputs: None.
   // Outputs: None.
   // Return Value: double - double value from token, -1 if outside of
   //                        range
   // Description: Parses token for an double within the min and max 
   //   values, if outside of range -1 is returned instead.
   //****************************************************************

   float get_float( char *token );
   //*****************************************************************
   // Params: token - string token
   // Inputs: None.
   // Outputs: None.
   // Return Value: float - floating point value from token, -1 if
   //                        outside of range
   // Description: Parses token for a float within the min[.001] and
   //   max[99.999] values, if outside of range -1 is returned instead.
   //****************************************************************

   char* get_name( char *token );
   //****************************************************************
   // Params: token - string token
   // Inputs: None.
   // Outputs: None.
   // Return Value: char - name
   //
   // Description: Gets a valid name from the token
   //
   //****************************************************************

   void parse_line( char *buffer );
   //****************************************************************
   // Params: char *buffer - 1 line of file
   // Inputs: None.
   // Outputs: Stores all loan info from current line to data members
   // Return Value: None
   //
   // Description: Parses line from report file.
   //
   //****************************************************************

   void add_loan( void );
   //****************************************************************
   // Params: None.
   // Inputs: Data Members storing record
   // Outputs: Creates Loan in Database
   // Return Value: None
   //
   // Description: Adds the loan if there is not currently a loan
   //              with the same loan id in the database.
   //****************************************************************

   void add_borrower( double, char* );
   //****************************************************************
   // Params: double - borrower id
   //         char*  - borrower name
   //
   // Inputs: Data Members storing record
   //
   // Outputs: Creates Borrower in the Database
   //
   // Return Value: None
   //
   // Description: Adds the borrower if there is not currently a 
   //              borrower with the same borrower id in the database.
   //****************************************************************

   void add_b_have_l( double, double );
   //****************************************************************
   // Params: double - borrower id
   //         double - loan id
   //
   // Inputs: Data Members storing record
   // Outputs: Creates Borrower_Have_Loan relationship in the DB.
   // Return Value: None
   //
   // Description: Adds the borrower have loans relation to the table
   //              if the borrower does not currently have the loan
   //****************************************************************

   void update_loan( void );
   //****************************************************************
   // Params: None.
   // Inputs: Data Members storing record
   // Outputs: Update loan in DB from Loan Status Report
   // Return Value: None
   //
   // Description: Feeds in new values from report to update a loan.
   //
   //****************************************************************

   void update_b_have_l( double, double );
   //****************************************************************
   // Params: None.
   // Inputs: Data Members storing record
   // Outputs: Update borrower have loan relationship for non FMCO
   //          purchased loans
   // Return Value: None
   //
   // Description: Creates relationship between borrowers and non
   //              FMCO purchased loans for correct numbers during
   //              risk calculations
   //****************************************************************

   bool fix_info( void );
   //****************************************************************
   // Params: None.
   // Inputs: Data Members storing record
   // Outputs: Data Members storing record
   // Return Value: bool- true- fixed
   //                     false- not fixed
   //
   // Description: Brings up Dialog Box with everything the parser
   //              could understand and allows user to modify info.
   //
   //****************************************************************

   bool check_lender( void );
   //****************************************************************
   // Params: None.
   // Inputs: Data Members storing record
   // Outputs: Data Members storing record
   // Return Value: bool- true- lender is in DB
   //                     false- lender is not in DB
   //
   // Description: Checks to see if Lender ID listed for loan is in
   //              the Lender DB, if it is returns true.  If it is not
   //              an Add Lender Dialog appears to allow lender to be
   //              added on fly.  If it not added it returns false.
   //
   //****************************************************************

};
#endif
